package main;

import java.util.ArrayList;
import java.util.List;

import design.Block;
import design.Property;
import design.State;
import timeSeries.TimeSeriesDatabase;

public class Main {
	private static TimeSeriesDatabase db = null;

	public static void main(String[] args) {

		setUpDatabase("./lib/Daten_156.csv", false, 0);
		System.out.println("single");
		singleStream("./lib/realStates_156.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_156.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_156.csv");

		db = null;
		setUpDatabase("./lib/Daten_1560.csv", false, 0);
		System.out.println("single");
		singleStream("./lib/realStates_1560.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_1560.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_1560.csv");
		
		// long run 15600
		db = null;
		setUpDatabase("./lib/Daten_1560.csv", true, 9);
		System.out.println("single");
		singleStream("./lib/realStates_1560.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_1560.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_1560.csv");

		// long run 156000
		db = null;
		setUpDatabase("./lib/Daten_1560.csv", true, 99);
		System.out.println("single");
		singleStream("./lib/realStates_1560.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_1560.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_1560.csv");
		
		// long run 1560000
		db = null;
		setUpDatabase("./lib/Daten_1560.csv", true, 900);
		System.out.println("single");
		singleStream("./lib/realStates_1560.csv");
		System.out.println("three");
		threeStreams("./lib/realStates_1560.csv");
		System.out.println("five");
		fiveStreams("./lib/realStates_1560.csv");

	}

	private static void setUpDatabase(String filenameData, boolean longRun, int timespan) {
		if (db == null) {
			// set up Database and Connection
			db = new TimeSeriesDatabase();
			// set up Data in TSDB
			db.setUpData(filenameData, longRun, timespan);
		}
	}

	private static void singleStream(String filenameReal) {
		// Test State Identification
		Evaluation eval = new Evaluation();
		evaluate(eval, db, setUpDataSingleStream());

	}

	private static void threeStreams(String filenameReal) {
		// Test State Identification
		Evaluation eval = new Evaluation();		
		evaluate(eval, db, setUpDataThreeStreams());

	}

	private static void fiveStreams(String filenameReal) {
		// Test State Identification
		Evaluation eval = new Evaluation();
		evaluate(eval, db, setUpDataFiveStreams());

	}

	private static void evaluate(Evaluation eval, TimeSeriesDatabase db, Block roboticArm) {
		// Different Variation Ranges
		for (double i = 0; i <= 0.4; i = i + 0.01) {
			eval.testPerformance(db, roboticArm, i);
		}
	}

	private static Block setUpDataSingleStream() {
		// setUp Information for state DriveDown
		Property p1 = new Property("gp", 1.5);

		List<Property> props1 = new ArrayList<Property>();
		props1.add(p1);
		State s1 = new State("DriveDown", props1);

		// setUp Information for state PickUp
		Property p2 = new Property("gp", -0.4);
		List<Property> props2 = new ArrayList<Property>();
		props2.add(p2);
		State s2 = new State("PickUp", props2);

		// List states
		List<State> states = new ArrayList<State>();
		states.add(s1);
		states.add(s2);

		// SetUp Block
		Block roboticArm = new Block("roboticArm", states);
		return roboticArm;
	}

	private static Block setUpDataThreeStreams() {
		// setUp Information for state DriveDown
		Property p1 = new Property("bp", 0);
		Property p2 = new Property("map", 1.5);
		Property p3 = new Property("gp", 1.5);

		List<Property> props1 = new ArrayList<Property>();
		props1.add(p1);
		props1.add(p2);
		props1.add(p3);
		State s1 = new State("DriveDown", props1);

		// setUp Information for state PickUp
		Property p4 = new Property("bp", 0);
		Property p5 = new Property("map", 1.5);
		Property p6 = new Property("gp", -0.4);

		List<Property> props2 = new ArrayList<Property>();
		props2.add(p4);
		props2.add(p5);
		props2.add(p6);
		State s2 = new State("PickUp", props2);

		// List states
		List<State> states = new ArrayList<State>();
		states.add(s1);
		states.add(s2);

		// SetUp Block
		Block roboticArm = new Block("roboticArm", states);
		return roboticArm;
	}

	private static Block setUpDataFiveStreams() {
		// setUp Information for state DriveDown
		Property p1 = new Property("bp", 0);
		Property p2 = new Property("map", 1.5);
		Property p3 = new Property("gp", 1.5);
		Property p4 = new Property("sap", -0.12);
		Property p5 = new Property("wp", 0);

		List<Property> props1 = new ArrayList<Property>();
		props1.add(p1);
		props1.add(p2);
		props1.add(p3);
		props1.add(p4);
		props1.add(p5);
		State s1 = new State("DriveDown", props1);

		// setUp Information for state PickUp
		Property p6 = new Property("bp", 0);
		Property p7 = new Property("map", 1.5);
		Property p8 = new Property("gp", -0.4);
		Property p9 = new Property("sap", -0.12);
		Property p10 = new Property("wp", 0);
		List<Property> props2 = new ArrayList<Property>();
		props2.add(p6);
		props2.add(p7);
		props2.add(p8);
		props2.add(p9);
		props2.add(p10);
		State s2 = new State("PickUp", props2);

		// List states
		List<State> states = new ArrayList<State>();
		states.add(s1);
		states.add(s2);

		// SetUp Block
		Block roboticArm = new Block("roboticArm", states);
		return roboticArm;
	}

}
